/* Sistemas Integrados TECNISA
   Planejamento Alimentao Industrial

Nome: PREVISAO_DE_CONSUMO_PLA

Descrio:
Clculo da Previso de Consumo atravs da stored procedure
CONSUMO_PLANEJADO e gravao na tabela PREVCONS.

P_CDFILIAL     - Codigo da Filial
P_SERVICOS     - String contendo sequencia de Servios a serem calculados
P_DTINICIAL    - Data inicial
P_DTFINAL      - Data final
P_PEACRESCIMO  - Percentual de acrscimo
P_CDPRODUTOINI - Cdigo do Produto inicial
P_CDPRODUTOFIN - Cdigo do Produto final
P_CDOPERADOR   - Cdigo do operador que solicitou o clculo
P_DTHRCALCPC   - Data/hora do clculo
P_NRCONSPLAN   - Nmero sequencial a ser utilizado para gravao dos dados
P_STDTINICIAL  - Lista de data inicial em string por servio
P_STDTFINAL    - Lista de data final em string por servio

Criao:
20/07/98 - Ederson

*/

IF EXISTS(SELECT name FROM sysobjects
          WHERE name = 'PREVISAO_DE_CONSUMO_PLA' AND type = 'P')
  DROP PROCEDURE PREVISAO_DE_CONSUMO_PLA
  GO

CREATE PROCEDURE PREVISAO_DE_CONSUMO_PLA
   @P_CDFILIAL     VARCHAR(4),
   @P_SERVICOS     VARCHAR(2000),
   @P_DTINICIAL    DATETIME,
   @P_DTFINAL      DATETIME,
   @P_PEACRESCIMO  NUMERIC(15,6),
   @P_CDPRODUTOINI VARCHAR(10),
   @P_CDPRODUTOFIN VARCHAR(10),
   @P_CDOPERADOR   VARCHAR(12),
   @P_DTHRCALCPC   DATETIME,
   @P_NRCONSPLAN   VARCHAR(10),
   @P_IDARREDOND   VARCHAR(1),
   @P_STDTINICIAL  VARCHAR(2000) = NULL,
   @P_STDTFINAL    VARCHAR(2000) = NULL
AS
   DECLARE @V_COUNTSERVICO  INT;
   DECLARE @V_CDSERVFILI    VARCHAR(5);
   DECLARE @V_CDARVINI      VARCHAR(20);
   DECLARE @V_CDARVFIN      VARCHAR(20);
   DECLARE @V_DTFINAL       DATETIME;
   DECLARE @V_DTINICIAL     DATETIME;
   DECLARE @V_CDFILIAL      VARCHAR(4);
   DECLARE @V_CDPRODUTO     VARCHAR(10);
   DECLARE @V_DTPREVCONS    DATETIME;

   DECLARE @V_IDCOMCUSPERC VARCHAR(01);
   DECLARE @V_IDCALPADCMP  VARCHAR(01);
   DECLARE @V_CDTPCUSTCMP  VARCHAR(02);
   DECLARE @V_NRTPCUSTCMP  INT;
   DECLARE @V_VRINDCALCMP  NUMERIC(15, 5);
   DECLARE @V_QTCOMES      NUMERIC(15, 5);
   DECLARE @V_VRCUSTO      NUMERIC(15, 5);
   DECLARE @V_VRCUSTOPERC  NUMERIC(15, 5);
   DECLARE @V_VRCUSTOPADR  NUMERIC(15, 5);

   DECLARE @V_CUR_CDFILIAL   VARCHAR(4);
   DECLARE @V_CUR_CDSERVFILI VARCHAR(5);

   DECLARE @V_IDUTCARDPADR VARCHAR(1);
   DECLARE @V_CDPRODUTOINI VARCHAR(10);
   DECLARE @V_CDPRODUTOFIN VARCHAR(10);
   DECLARE @V_CDSERVDEPEND VARCHAR(05);
BEGIN
   SET @V_CDARVINI = @P_CDPRODUTOINI;
   SET @V_CDARVFIN = SUBSTRING(@P_CDPRODUTOFIN + REPLICATE('z', 20), 1, 20);

   SELECT @V_IDUTCARDPADR = MAX(IDUTCARDPADR)
   FROM PARAMGERAL;
   
   IF (@V_IDUTCARDPADR = 'S') BEGIN
      SELECT @V_CDSERVDEPEND = MAX(CDSERVFILI)
        FROM SERVICO
       WHERE CDFILIAL = @P_CDFILIAL
         AND IDCARDDESCENTR = 'S';

       IF @V_CDSERVDEPEND IS NOT NULL BEGIN
          SET @V_IDUTCARDPADR = 'D';
       END;
   END;
   
   IF (@V_IDUTCARDPADR = 'S') BEGIN
      SELECT @V_CDFILIAL = MAX(CDFILIAL)
      FROM   FILIAL
      WHERE  IDFILCENTPME = 'S' AND
             CDFILIAL = @P_CDFILIAL AND
             CDFILCENTPME IS NULL;
      IF @V_CDFILIAL IS NOT NULL BEGIN
         RETURN;
      END;

      SELECT @V_CDSERVFILI = MIN(A.CDSERVFILI)
      FROM
         (SELECT
             S.CDSERVFILI,
             ISNULL(C.IDCARDLIBAGRE, 'N') AS IDCARDLIBAGRE
          FROM
             SERVICO S LEFT JOIN CARDAMES C ON
                            S.CDFILIAL     = C.CDFILIAL   AND
                            S.CDSERVFILI   = C.CDSERVFILI AND
                            C.DTMESCARDAPIO BETWEEN @P_DTINICIAL AND @P_DTFINAL
          WHERE
             S.CDFILIAL = @P_CDFILIAL AND
             ((CHARINDEX (S.CDSERVFILI, @P_SERVICOS) <> 0) OR (@P_SERVICOS = 'T'))
             AND IDTIPOSERV = 'N') A
      WHERE  A.IDCARDLIBAGRE = 'N';
      IF (@V_CDSERVFILI IS NOT NULL) BEGIN
         RETURN(1); /*Cardpio no liberado para produo*/
      END;
   END;

   SELECT @V_CDPRODUTOINI = MAX(CDPRODUTO) FROM PRODUTO WHERE CDARVPROD = @P_CDPRODUTOINI;
   SELECT @V_CDPRODUTOFIN = MAX(CDPRODUTO) FROM PRODUTO WHERE CDARVPROD = @P_CDPRODUTOFIN;

   /* - REGISTRO DO LOG E EXCLUSO DOS REGISTROS ANTERIORES DA PREVCONS ----- */
   SET @V_COUNTSERVICO = 2;
   DECLARE C_SERVICOS CURSOR FOR
      SELECT
         CDSERVFILI
      FROM
         SERVICO
      WHERE
         (CDFILIAL   = @P_CDFILIAL) AND
         (IDSERVATIV = 'S')         AND
         ((CHARINDEX(CDSERVFILI, @P_SERVICOS) > 0) OR (@P_SERVICOS = 'T')) 
      ORDER BY
           CDSERVFILI;

   OPEN C_SERVICOS;

   FETCH NEXT FROM  C_SERVICOS INTO @V_CDSERVFILI;
   WHILE (@@FETCH_STATUS = 0)
   BEGIN
      IF @P_STDTINICIAL IS NULL
      BEGIN
        SET @V_DTINICIAL  = @P_DTINICIAL;
        SET @V_DTFINAL    = @P_DTFINAL;
      END
      ELSE
      BEGIN

         SET @V_DTINICIAL  = CONVERT(DATETIME,SUBSTRING(@P_STDTINICIAL, @V_COUNTSERVICO, 8),103);
         SET @V_DTFINAL    = CONVERT(DATETIME,SUBSTRING(@P_STDTFINAL, @V_COUNTSERVICO, 8), 103);

        IF DATALENGTH(@P_STDTINICIAL) > 10
        BEGIN
           SET @V_COUNTSERVICO = @V_COUNTSERVICO + 9;
        END;
      END;

      INSERT INTO LOGPREVCONS
        (CDFILIAL,
         CDOPERADOR,
         DTHRCALCPC,
         NRCONSPLANPC,
         CDSERVFILI,
         DTINICIALPC,
         DTFINALPC,
         CDPRODINICPC,
         CDPRODFINAPC,
         PEACRESCIMPC)
      VALUES
        (@P_CDFILIAL,
         @P_CDOPERADOR,
         @P_DTHRCALCPC,
         @P_NRCONSPLAN,
         @V_CDSERVFILI,
         @V_DTINICIAL,
         @V_DTFINAL,
         @V_CDPRODUTOINI,
         @V_CDPRODUTOFIN,
         @P_PEACRESCIMO);
      /* - APAGA OS REGISTROS ANTERIORES DA TABELA PREVCONS ----------------- */
      DELETE FROM PREVCONS
      WHERE (CDFILIAL   = @P_CDFILIAL)                              AND
            (CDSERVFILI = @V_CDSERVFILI)                            AND
            (DTPREVCONS BETWEEN @V_DTINICIAL    AND @V_DTFINAL)     AND
            (EXISTS (SELECT CDPRODUTO
                            FROM PRODUTO
                            WHERE CDARVPROD BETWEEN @V_CDARVINI AND @V_CDARVFIN
                              AND PREVCONS.CDPRODUTO = PRODUTO.CDPRODUTO));

   FETCH NEXT FROM C_SERVICOS INTO @V_CDSERVFILI;
   END; /* LOOP DOS SERVIOS */

   CLOSE C_SERVICOS;
   DEALLOCATE C_SERVICOS;

   EXEC CONSUMO_PLANEJADO_PLA    @P_CDFILIAL,
                                 @P_SERVICOS,
                                 @P_DTINICIAL,
                                 @P_DTFINAL,
                                 'C',   /* GRAVAO NA TABELA CONSPLAN */
                                 'CFP', /* CARDPIO, CONSUMO FIXO E PERCAPITA */
                                 'N',   /* NO SEPARAR POR SETOR */
                                 @P_PEACRESCIMO,
                                 @P_IDARREDOND,
                                 @P_NRCONSPLAN,
                                 @V_CDARVINI,
                                 @V_CDARVFIN,
                                 @P_CDOPERADOR,
                                 @P_STDTINICIAL,
                                 @P_STDTFINAL;

   SELECT @V_IDCALPADCMP = MAX(IDCALPADCMP),
          @V_CDTPCUSTCMP = MAX(CDTPCUSTCMP),
          @V_VRINDCALCMP = MAX(VRINDCALCMP)
   FROM PARAMPLANEJA
   WHERE (CDFILIAL = @P_CDFILIAL);

   SET @V_NRTPCUSTCMP = CONVERT(INTEGER, @V_CDTPCUSTCMP);

   SELECT @V_IDCOMCUSPERC = MAX(IDCOMCUSPERC)
   FROM PARAMFILIAL
   WHERE (CDFILIAL = @P_CDFILIAL);

   IF (@V_IDCALPADCMP = 'S') BEGIN
      DECLARE CUR_SERVICO CURSOR FOR
         SELECT CDFILICPLA AS CDFILIAL,
                 CDSERVCPLA AS CDSERVFILI
         FROM CONSPLAN
         WHERE
            (CDFILICPLA = @P_CDFILIAL) AND
            (NRCONSPLAN = @P_NRCONSPLAN)
         GROUP BY CDFILICPLA,
                  CDSERVCPLA;

      OPEN CUR_SERVICO;
      FETCH NEXT FROM CUR_SERVICO INTO @V_CUR_CDFILIAL,
                                       @V_CUR_CDSERVFILI;
      WHILE (@@FETCH_STATUS = 0)
      BEGIN
         SELECT @V_VRCUSTO = SUM(TEKNISA.CUSTO_PRODUTO(CONSPLAN.CDFILICPLA,
                                                       CONSPLAN.CDPRODCPLA,
                                                       @V_NRTPCUSTCMP,
                                                       CONSPLAN.DTCONSPLAN,
                                                       GETDATE()) * CONSPLAN.QTCONSPLAN)
         FROM CONSPLAN
         WHERE (CONSPLAN.NRCONSPLAN = @P_NRCONSPLAN)     AND
               (CONSPLAN.CDFILICPLA = @V_CUR_CDFILIAL)   AND
               (CONSPLAN.CDSERVCPLA = @V_CUR_CDSERVFILI);

         SELECT @V_VRCUSTOPADR = SUM(ISNULL(VRCUSTPADR * @V_VRINDCALCMP, 0.0))
         FROM CUSTPADR
         WHERE
            (CDFILIAL   = @V_CUR_CDFILIAL)   AND
            (CDSERVFILI = @V_CUR_CDSERVFILI) AND
            (CONVERT(VARCHAR(2), DATEPART(mm, DTREFCUSPADR))+CONVERT(VARCHAR(4), DATEPART(YYYY, DTREFCUSPADR)) =
             CONVERT(VARCHAR(2), DATEPART(mm, @P_DTFINAL))+CONVERT(VARCHAR(4), DATEPART(YYYY, @P_DTFINAL)))

         SELECT
            @V_QTCOMES = SUM(CASE @V_IDCOMCUSPERC
                                  WHEN 'C' THEN QTCLIESTICAR
                                  ELSE QTCOMESTICAR END)
         FROM
            CARDADIA
         WHERE
            (CDFILIAL   = @V_CUR_CDFILIAL)   AND
            (CDSERVFILI = @V_CUR_CDSERVFILI) AND
            (DTCARDAPIO BETWEEN @P_DTINICIAL AND @P_DTFINAL);

         SET @V_VRCUSTOPERC = 0;
         IF (@V_QTCOMES <> 0.0) AND (@V_VRCUSTO <> 0.0) BEGIN
            SET @V_VRCUSTOPERC = (@V_VRCUSTO/@V_QTCOMES);
         END;

         IF (@V_VRCUSTOPADR > @V_VRCUSTOPERC) BEGIN
            /* INSERE NA TABELA PREVCONS AS INFORMAES CALCULADAS NA CONSPLAN */
            INSERT INTO PREVCONS
               (CDFILIAL,
                CDSERVFILI,
                CDPRODUTO,
                DTPREVCONS,
                QTPREVCONS,
                CDOPERADOR)
            SELECT
               CDFILICPLA,
               CDSERVCPLA,
               CDPRODCPLA,
               DTCONSPLAN,
               SUM(QTCONSPLAN),
               @P_CDOPERADOR
            FROM CONSPLAN
            WHERE(CDFILICPLA = @P_CDFILIAL)       AND
                 (NRCONSPLAN = @P_NRCONSPLAN)     AND
                 (CDFILICPLA = @V_CUR_CDFILIAL)   AND
                 (CDSERVCPLA = @V_CUR_CDSERVFILI) AND 
                 EXISTS (SELECT CDSERVCPLA
                           FROM SERVICO
                          WHERE CONSPLAN.CDFILICPLA = SERVICO.CDFILIAL
                            AND CONSPLAN.CDSERVCPLA = SERVICO.CDSERVFILI
                            AND SERVICO.IDSERVATIV = 'S')
            GROUP BY
               CDFILICPLA,
               CDSERVCPLA,
               CDPRODCPLA,
               DTCONSPLAN;
         END;
         FETCH NEXT FROM CUR_SERVICO INTO  @V_CUR_CDFILIAL, @V_CUR_CDSERVFILI;
      END;
      CLOSE CUR_SERVICO;
      DEALLOCATE CUR_SERVICO;
   END
   ELSE BEGIN
      /* INSERE NA TABELA PREVCONS AS INFORMAES CALCULADAS NA CONSPLAN */
      INSERT INTO PREVCONS
         (CDFILIAL,
          CDSERVFILI,
          CDPRODUTO,
          DTPREVCONS,
          QTPREVCONS,
          CDOPERADOR)
        SELECT
            CDFILICPLA,
            CDSERVCPLA,
            CDPRODCPLA,
            DTCONSPLAN,
            SUM(QTCONSPLAN),
            @P_CDOPERADOR
        FROM
            CONSPLAN
        WHERE
             (CDFILICPLA = @P_CDFILIAL)   AND
             (NRCONSPLAN = @P_NRCONSPLAN) AND 
             EXISTS (SELECT CDSERVCPLA
                       FROM SERVICO
                      WHERE CONSPLAN.CDFILICPLA = SERVICO.CDFILIAL
                        AND CONSPLAN.CDSERVCPLA = SERVICO.CDSERVFILI
                        AND SERVICO.IDSERVATIV = 'S')
        GROUP BY
             CDFILICPLA,
             CDSERVCPLA,
             CDPRODCPLA,
             DTCONSPLAN;
   END;

   /* APAGA OS DADOS DA TABELA DE TRABALHO UTILIZADA NO CLCULO */

   DELETE FROM CONSPLAN
   WHERE
        CDFILICPLA = @P_CDFILIAL AND
        NRCONSPLAN = @P_NRCONSPLAN;
END;
GO

-- VERSAO --
EXEC ATUVEROBJ 'PREVISAO_DE_CONSUMO_PLA','5.70.000';
GO