import {
  Http, IDictionary, Metadata, IEventParam, Loader,
} from '@zeedhi/core';
import {
  Text, ApexChart, Modal, ModalService, Select, Date as DateInput,
} from '@zeedhi/common';
import { IterableComponentRender, ICardTitle } from '@zeedhi/teknisa-components-common';

export class DashboardSupervisorController {
  private basicModal!: Modal;

  private modalResumoTarefas!: Modal;

  public hierarquiasController = Loader.getInstance('HierarquiasController');

  // responsável por armazenar os dados formatados de local e data que são
  // enviados ao acionar o filtro dos cards
  private dataFilter: IDictionary = {
    selectLocal: '',
    selectData: '',
  };

  // responsavel por requisitar os dados e popular o iterable do card ultimas tarefas realizadas
  public async getDadosUltimasTarefas() {
    const hierarchyUsersList = await this.hierarquiasController.mountHierarchyUsersArray();

    Http.post('/getDadosUltimasTarefas', { param1: this.dataFilter, param2: hierarchyUsersList }).then((response: IDictionary) => {
      const { dados } = response.data.dataset;
      const iterable = Metadata.getInstance<IterableComponentRender>('iterableUltimasTarefas');
      dados.forEach((data: IDictionary) => {
        iterable.datasource.data.push({
          CODIGO: data.CODIGO,
          NOME: data.NOME,
          TEMPO: ` Finalizada dia ${data.DTULTATU}`,
          LOCAL: `${data.NMLOCALCLIENTE} | ${data.NMLOCALPAI}`,
          FINALIZADO: data.DTULTATU,
          STATUS: data.STATUS,
        });
      });
    });
  }

  // responsavel por requisitar os dados do card Resumo de tarefas, chamar a função de populaIterable,
  // mudar o texto da segunda coluna do card de acordo com a data
  public async getDadosResumoTarefas() {
    const hierarchyUsersList = await this.hierarquiasController.mountHierarchyUsersArray();

    const dataTime = new Date();
    const TarefaAtrasada = 'Tarefa Atrasada';
    const VistoriaEmAndamento = 'Vistoria em andamento';
    const TarefaPendente = 'Tarefa Pendente';
    const TarefaEmAndamento = 'Tarefa em andamento';
    Http.post('/getDadosResumoTarefas', { param1: this.dataFilter, param2: hierarchyUsersList }).then((response: IDictionary) => {
      const { dados } = response.data.dataset;
      dados.forEach((data: IDictionary) => {
        const dataBanco = data.DTULTATU;
        if (dataBanco != null) {
          const arr = dataBanco.split('/');
          const invertePosicao = arr.reverse();
          const formata = invertePosicao.join('/');
          const dataForm = new Date(formata);
          if (dataForm < dataTime) {
            if (data.STATUS === TarefaAtrasada) {
              const texto = `Atrasado a ${data.DIAS} dias`;
              this.populaIterable(texto, data);
            } else if (data.STATUS === TarefaPendente) {
              const texto = ` Pendente a ${data.DIAS} dias`;
              this.populaIterable(texto, data);
            } else if (data.STATUS === VistoriaEmAndamento) {
              const texto = ` Iniciada dia ${data.DTULTATU} `;
              this.populaIterable(texto, data);
            } else if (data.STATUS === TarefaEmAndamento) {
              const texto = ` Iniciada dia ${data.DTULTATU} `;
              this.populaIterable(texto, data);
            } else {
              const texto = `Finalizada dia ${data.DTULTATU}`;
              this.populaIterable(texto, data);
            }
          } else {
            const texto = `Finalizada dia ${data.DTULTATU}`;
            this.populaIterable(texto, data);
          }
        } else {
          const texto = 'Aguardando....';
          this.populaIterable(texto, data);
        }
      });
    });
  }

  // responsável por alterar a fonte cor de fundo onde esta o status do card Resumo de tarefa
  public stylizeStatus({ component }: IEventParam<Text>) {
    if (component.text === 'Vistoria Realizada') {
      component.text = "<span  style='background-color: #dee2e6; color: green; font-size: 12px;'>VISTORIA&nbsp;CONCLUÍDA</span>";
    }
    if (component.text === 'Tarefa Concluída') {
      component.text = "<span  style='background-color: #dee2e6; color: green; font-size: 12px;'>TAREFA&nbsp; CONCLUÍDA</span>";
    }
    if (component.text === 'Tarefa em andamento') {
      component.text = "<span  style='background-color: #dee2e6; color: #023e8a; font-size: 12px;'>TAREFA:&nbsp;EM&nbsp;ANDAMENTO</span>";
    }
    if (component.text === 'Vistoria em andamento') {
      component.text = "<span  style='background-color: #dee2e6; color: #023e8a; font-size: 12px;'>VISTORIA:&nbspEM&nbsp;ANDAMENTO</span>";
    }
    if (component.text === 'Tarefa Cancelada') {
      component.text = "<span  style='background-color: #dee2e6; color: black; font-size: 12px;'>TAREFA&nbsp;CANCELADA</span>";
    }
    if (component.text === 'Tarefa Atrasada') {
      component.text = "<span  style='background-color: #dee2e6; color: red; font-size: 12px;'>TAREFA:&nbsp;ATRASADA</span>";
    }
    if (component.text === 'Aguardando vistoria') {
      component.text = "<span  style='background-color:#dee2e6; color: #ff6700; font-size: 12px;';'>AGUARDANDO&nbsp;VISTORIA</span>";
    }
    if (component.text === 'Tarefa Pendente') {
      component.text = "<span  style='background-color:#dee2e6; color: #ff6700; font-size: 12px;'>TAREFA&nbsp; PENDENTE</span>";
    }
  }

  // responsavel por inserir os dados do Iterable Resumo de Tarefas
  public populaIterable(texto: string, data: IDictionary) {
    const iterableResumoTarefa = Metadata.getInstance<IterableComponentRender>('iterableResumoTarefa');
    iterableResumoTarefa.datasource.data.push({
      CODIGO: data.CODIGO,
      NOME: data.NOME,
      TEMPO: `${texto}`,
      LOCAL: `${data.NMLOCALCLIENTE} | ${data.NMLOCALPAI}`,
      FINALIZADO: `${texto}`,
      STATUS: data.STATUS,
    });
  }

  // responsavel por pegar os dados do chard Tarefas Realizadas e plotar as informações
  public async getDadosChartTarefasRealizadas() {
    const hierarchyUsersList = await this.hierarquiasController.mountHierarchyUsersArray();

    const selectGrafico = Metadata.getInstance<ApexChart>('chartTarefaRealizadas'); // Futuramente mudar para post devido ao tamanho do array de usuários
    Http.post('/getDadosChartTarefas', { param: hierarchyUsersList }).then((response: IDictionary) => {
      const { dados } = response.data.dataset;
      if (dados.length) {
        dados.forEach((data: IDictionary) => {
          selectGrafico.options.labels.push(data.NMTIPOLOCALCLI);
          (selectGrafico.series as any[]).push(data.QUANTIDADE);
        });
      } else {
        selectGrafico.options.labels.push('Não há dados disponíveis');
        (selectGrafico.series as any[]).push(1);
      }
    });
  }

  // responsavel por pegar os dados do chard Não Conformidades e plotar as informações
  public async getDadosChartNaoConformidade() {
    const hierarchyUsersList = await this.hierarquiasController.mountHierarchyUsersArray();

    const data: any = [];
    Http.post('/getDadosChartNaoConformidades', { param: hierarchyUsersList }).then((response: IDictionary) => {
      const { dados } = response.data.dataset;
      const selectGrafico = Metadata.getInstance<ApexChart>('bar-chart');
      if (dados.length) {
        dados.forEach((dadosGraf: IDictionary) => {
          data.push({
            name: dadosGraf.NMTIPOLOCALCLI,
            data: [dadosGraf.QUANTIDADE],
          });
        });
        selectGrafico.series = data;
      } else {
        selectGrafico.options.labels.push('Não há dados disponíveis');
        selectGrafico.series = [{ data: 100 }];
      }
    });
  }

  // responsavel por pegar os dados do chard Progresso Diário e plotar as informações
  public async getDadosChartProgressoDiario() {
    const hierarchyUsersList = await this.hierarquiasController.mountHierarchyUsersArray();

    let dataFormatada = '';
    const dataTime = new Date();
    const ano = dataTime.getFullYear();
    const mes = dataTime.getUTCMonth();
    const dia = dataTime.getDate();
    if (mes < 10) {
      const mesFormatado = `${mes + 1}`;
      dataFormatada = `${dia}/0${mesFormatado}/${ano}`;
    } else {
      dataFormatada = `${dia}/${mes + 1}/${ano}`;
    }
    const selectObjectData = Metadata.getInstance<ICardTitle>('cardProgressoDiario');
    selectObjectData.subtitle = dataFormatada;
    const da: any = [];
    Http.get(`/getDadosChartProgressoDiario?hierarchyUsersList=${hierarchyUsersList}`).then((response: IDictionary) => {
      const { dados } = response.data.dataset;
      const selectGrafico = Metadata.getInstance<ApexChart>('bar-horizontal-chart');
      if (dados.length) {
        dados.forEach((dadosGraf: IDictionary) => {
          selectGrafico.options.labels.push(dadosGraf.NMLOCALCLIENTE);
          (selectGrafico.series as any[]).push(dadosGraf.PORCENTAGEM);
        });
      } else {
        selectGrafico.options.labels.push('Não há dados disponíveis');
        selectGrafico.series = da;
      }
    });
  }

  // cria os dois modais responsaveis pelos filtros dos cards
  public async createBasicUsageModal() {
    this.basicModal = await ModalService.createFromJson('modal/modalFiltroDashboardUtimasTarefas', true);
    this.modalResumoTarefas = await ModalService.createFromJson('modal/modalFiltroDashboardResumoTarefas', true);
  }

  // abre o modal ultimas tarefas
  public openBasicModal() {
    this.basicModal.show();
  }

  // abre o modal resumo de tarefas
  public openModalResumoTarefas() {
    this.modalResumoTarefas.show();
  }

  // fecha o modal ultimas tarefas
  public closeBasicModal() {
    this.basicModal.hide();
  }

  // fecha o modal resumo de tarefa
  public closeModalResumoTarefas() {
    this.modalResumoTarefas.hide();
  }

  // instancia e pega o valor do select de local - Ultimas tarefas realizadas
  get getLocalModalUltimasTarefas() {
    return Metadata.getInstance<Select>('selectLocalUltimasTarefas').value || '';
  }

  // instancia e pega o valor da data filtro ultimas taredas realizadas
  get getDataModalUltimasTarefas() {
    const dataDe = Metadata.getInstance<DateInput>('dataDeUltimasTarefas').value;
    return dataDe;
  }

  // instancia e pega o valor do select de local - resumo de tarefas
  get getLocalModalResumoTarefa() {
    return Metadata.getInstance<Select>('selectLocalResumoTarefa').value || '';
  }

  // instancia e pega o valor da data filtro resumo de tarefa
  get getDataModalResumoTarefa() {
    const dataDe = Metadata.getInstance<DateInput>('dataDeResumoTarefa').value;
    return dataDe;
  }

  // metodo responsavel pelo filtro do card ultimas tarefas
  public filterCardUltimasTarefas() {
    this.dataFilter.selectLocal = this.getLocalModalUltimasTarefas;
    this.dataFilter.selectData = this.getDataModalUltimasTarefas;
    const iterable = Metadata.getInstance<IterableComponentRender>('iterableUltimasTarefas');
    iterable.datasource.data = [];
    this.getDadosUltimasTarefas();
    this.closeBasicModal();
  }

  // metodo responsavel pelo filtro do card resumo de tarefas
  public filterCardResumoTarefas() {
    this.dataFilter.selectLocal = this.getLocalModalResumoTarefa;
    this.dataFilter.selectData = this.getDataModalResumoTarefa;
    const iterableTarefaDiaria = Metadata.getInstance<IterableComponentRender>('iterableResumoTarefa');
    iterableTarefaDiaria.datasource.data = [];
    this.getDadosResumoTarefas();
    this.closeModalResumoTarefas();
  }
}
